export type MetadataDefinable<T> = {new(target: Function): T};

const getClassMetadata = <TFunction extends Function, DefinitionType>(target: TFunction | Object,
                                                                      metadataKey: any,
                                                                      definitionType: MetadataDefinable<DefinitionType>,
                                                                      isCreate: boolean = false): DefinitionType => {

  let clazz = target instanceof Function ? target.prototype : target;
  let definition: DefinitionType;
  if (Reflect.hasOwnMetadata(metadataKey, clazz) !== true) {
      if (!isCreate) {
          return undefined;
      }
      definition = Object.create(definitionType.prototype);
      definition.constructor.call(definition, clazz);

      Reflect.defineMetadata(metadataKey, definition, clazz);
  } else {
    definition = Reflect.getMetadata(metadataKey, clazz);
  }
  return definition;
};

export default getClassMetadata;
